"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.invalidateCache = exports.get = exports.set = exports.remove = exports.init = undefined;

var _regenerator = require("babel-runtime/regenerator");

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require("babel-runtime/helpers/asyncToGenerator");

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _extends2 = require("babel-runtime/helpers/extends");

var _extends3 = _interopRequireDefault(_extends2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * SSR Memcached
 * -------------
 * 
 * This is a simple in-memory cache layer applied to the server side rendering.
 * It is suitable for small websites with a limited amount of contents.
 * 
 * NOTE: this uses the server's RAM, it's a limited resource, use it carefully!
 * 
 * ### getCacheKey
 * 
 * this function is supposed to return an object that contains the key value plus
 * some optional meta data:
 * 
 *     {
 *        value: 'ojdoiejwow',
 *        sessionId: 22,
 *        ...
 *     }
 * 
 * The `value` should be a unique string identifier that is used to refer to the cache
 * record in a map (the `cache` map). Other meta data may be used to apply a cache
 * invalidation strategy. For instance it is usually convenient to invalidate all the
 * cached contents that are related to a specific session on logout.
 * 
 * The default, simplicistic, approac is to use the `req.url` as key value. This will
 * work very well for a simple static website.
 * 
 * But if you are handling dynamic contents or session based contents (aka user login)
 * you might want to implement this function with your own custom business logic so to
 * be able to apply a fine grained cache invalidation.
 * 
 * ### invalidateCache(string|function)
 * 
 * If called with a string will invalidate a specific cache record by it's key.
 * 
 * If called with a function it will scan the `keys` index to invalidate multiple records
 * based on the stored key's meta data:
 * 
 *     invalidateCache(key => key.sessionId === 1)
 * 
 * This example will invalidate any stored cache record who's key's meta data "sessionId"
 * has the value of "1". This is normally applied on logout or when a specific content
 * change in the server.
 * 
 */
var defaultSettings = {
    canCache: function canCache(req, res) {
        return true;
    },
    getCacheKey: function getCacheKey(req, res) {
        return { value: req.url };
    },
    getCacheExpiry: function getCacheExpiry(req, res) {
        return 32503590000000;
    }, // year 3000, permanent default cache
    onCacheWrites: function onCacheWrites(req, res, html) {
        return html;
    },
    onCacheRead: function onCacheRead(req, res, _ref) {
        var cnt = _ref.cnt,
            exp = _ref.exp,
            ctm = _ref.ctm;
        return cnt;
    }
};

var settings = null;
var cache = {};
var keys = {};

var init = exports.init = function init() {
    var receivedSettings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    settings = (0, _extends3.default)({}, defaultSettings, receivedSettings);
};

// retrieve the request's cache keys and keeps up to date an index
// of all the keys that are in use. This could be used to offer
// an invalidation strategy that is based on the key meta data.
var getCacheKey = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(req, res) {
        var key;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        _context.next = 2;
                        return settings.getCacheKey(req, res);

                    case 2:
                        key = _context.sent;

                        keys[key.value] = key;
                        return _context.abrupt("return", key);

                    case 5:
                    case "end":
                        return _context.stop();
                }
            }
        }, _callee, undefined);
    }));

    return function getCacheKey(_x2, _x3) {
        return _ref2.apply(this, arguments);
    };
}();

var remove = exports.remove = function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
        var toBeRemoved = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

        var _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, key;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        if (!settings.serializer) {
                            _context2.next = 4;
                            break;
                        }

                        _context2.next = 3;
                        return settings.serializer.remove(toBeRemoved);

                    case 3:
                        return _context2.abrupt("return");

                    case 4:
                        _iteratorNormalCompletion = true;
                        _didIteratorError = false;
                        _iteratorError = undefined;
                        _context2.prev = 7;


                        for (_iterator = toBeRemoved[Symbol.iterator](); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                            key = _step.value;

                            cache[key] = null;
                            keys[key] = null;
                        }
                        _context2.next = 15;
                        break;

                    case 11:
                        _context2.prev = 11;
                        _context2.t0 = _context2["catch"](7);
                        _didIteratorError = true;
                        _iteratorError = _context2.t0;

                    case 15:
                        _context2.prev = 15;
                        _context2.prev = 16;

                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }

                    case 18:
                        _context2.prev = 18;

                        if (!_didIteratorError) {
                            _context2.next = 21;
                            break;
                        }

                        throw _iteratorError;

                    case 21:
                        return _context2.finish(18);

                    case 22:
                        return _context2.finish(15);

                    case 23:
                    case "end":
                        return _context2.stop();
                }
            }
        }, _callee2, undefined, [[7, 11, 15, 23], [16,, 18, 22]]);
    }));

    return function remove() {
        return _ref3.apply(this, arguments);
    };
}();

var set = exports.set = function () {
    var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(req, res, html) {
        var key, cacheObject;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        _context3.next = 2;
                        return settings.canCache(req, res);

                    case 2:
                        if (_context3.sent) {
                            _context3.next = 4;
                            break;
                        }

                        return _context3.abrupt("return", false);

                    case 4:
                        _context3.next = 6;
                        return getCacheKey(req, res);

                    case 6:
                        key = _context3.sent;
                        cacheObject = {
                            ctm: Date.now(),
                            exp: settings.getCacheExpiry(req, res),
                            cnt: settings.onCacheWrites(req, res, html)

                            // custom serializer implementation
                        };

                        if (!settings.serializer) {
                            _context3.next = 12;
                            break;
                        }

                        _context3.next = 11;
                        return settings.serializer.set(key, cacheObject);

                    case 11:
                        return _context3.abrupt("return");

                    case 12:

                        cache[key.value] = cacheObject;

                    case 13:
                    case "end":
                        return _context3.stop();
                }
            }
        }, _callee3, undefined);
    }));

    return function set(_x5, _x6, _x7) {
        return _ref4.apply(this, arguments);
    };
}();

var get = exports.get = function () {
    var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(req, res) {
        var key, cacheObject;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
            while (1) {
                switch (_context4.prev = _context4.next) {
                    case 0:
                        _context4.next = 2;
                        return getCacheKey(req, res);

                    case 2:
                        key = _context4.sent;

                        if (!settings.serializer) {
                            _context4.next = 9;
                            break;
                        }

                        _context4.next = 6;
                        return settings.serializer.get(key);

                    case 6:
                        _context4.t0 = _context4.sent;
                        _context4.next = 10;
                        break;

                    case 9:
                        _context4.t0 = cache[key.value];

                    case 10:
                        cacheObject = _context4.t0;

                        if (cacheObject) {
                            _context4.next = 13;
                            break;
                        }

                        return _context4.abrupt("return");

                    case 13:
                        if (!(cacheObject.exp < Date.now())) {
                            _context4.next = 17;
                            break;
                        }

                        _context4.next = 16;
                        return remove(key.value);

                    case 16:
                        return _context4.abrupt("return");

                    case 17:
                        return _context4.abrupt("return", settings.onCacheRead(req, res, cacheObject));

                    case 18:
                    case "end":
                        return _context4.stop();
                }
            }
        }, _callee4, undefined);
    }));

    return function get(_x8, _x9) {
        return _ref5.apply(this, arguments);
    };
}();

// apply a strategy to filter out some cache keys that need to be
// invalidated, based on some key meta data.
var invalidateCache = exports.invalidateCache = function invalidateCache(strategy) {
    try {
        var toBeRemoved = [];
        for (var key in keys) {
            if (keys[key] !== null && strategy(keys[key])) {
                toBeRemoved.push(key);
            }
        }
        return remove(toBeRemoved);
    } catch (err) {
        console.log("[ssr] invalidateCache - " + err.message);
    }
};